@@ -179,7 +179,7 @@ type tcMocks struct {
 	oneClick          *domocks.MockOneClickService
 	listen            *domocks.MockListenerService
 	monitoring        *domocks.MockMonitoringService
-	sandbox           *domocks.MockSandboxService
+	serverless        *domocks.MockServerlessService
 }
 
 func withTestClient(t *testing.T, tFn testFn) {
@@ -220,7 +220,7 @@ func withTestClient(t *testing.T, tFn testFn) {
 		oneClick:          domocks.NewMockOneClickService(ctrl),
 		listen:            domocks.NewMockListenerService(ctrl),
 		monitoring:        domocks.NewMockMonitoringService(ctrl),
-		sandbox:           domocks.NewMockSandboxService(ctrl),
+		serverless:        domocks.NewMockServerlessService(ctrl),
 	}
 
 	config := &CmdConfig{
@@ -237,10 +237,6 @@ func withTestClient(t *testing.T, tFn testFn) {
 
 		setContextAccessToken: func(token string) {},
 
-		checkSandboxStatus: func(c *CmdConfig) error {
-			return nil
-		},
-
 		Keys:              func() do.KeysService { return tm.keys },
 		Sizes:             func() do.SizesService { return tm.sizes },
 		Regions:           func() do.RegionsService { return tm.regions },
@@ -272,7 +268,7 @@ func withTestClient(t *testing.T, tFn testFn) {
 		OneClicks:         func() do.OneClickService { return tm.oneClick },
 		Apps:              func() do.AppsService { return tm.apps },
 		Monitoring:        func() do.MonitoringService { return tm.monitoring },
-		Sandbox:           func() do.SandboxService { return tm.sandbox },
+		Serverless:        func() do.ServerlessService { return tm.serverless },
 	}
 
 	tFn(config, tm)
