@@ -14,17 +14,21 @@ limitations under the License.
 package commands
 
 import (
+	"encoding/base64"
 	"encoding/json"
 	"errors"
+	"fmt"
+	"os"
 	"strings"
 
+	"github.com/apache/openwhisk-client-go/whisk"
 	"github.com/digitalocean/doctl"
 	"github.com/digitalocean/doctl/commands/displayers"
 	"github.com/digitalocean/doctl/do"
 	"github.com/spf13/cobra"
 )
 
-// Functions generates the sandbox 'functions' subtree for addition to the doctl command
+// Functions generates the serverless 'functions' subtree for addition to the doctl command
 func Functions() *Command {
 	cmd := &Command{
 		Command: &cobra.Command{
@@ -70,20 +74,141 @@ You can provide inputs and inspect outputs.`,
 	return cmd
 }
 
-// RunFunctionsGet supports the 'sandbox functions get' command
+// RunFunctionsGet supports the 'serverless functions get' command
 func RunFunctionsGet(c *CmdConfig) error {
 	err := ensureOneArg(c)
 	if err != nil {
 		return err
 	}
-	output, err := RunSandboxExec(actionGet, c, []string{flagURL, flagCode, flagSave}, []string{flagSaveEnv, flagSaveEnvJSON, flagSaveAs})
+	urlFlag, _ := c.Doit.GetBool(c.NS, flagURL)
+	codeFlag, _ := c.Doit.GetBool(c.NS, flagCode)
+	saveFlag, _ := c.Doit.GetBool(c.NS, flagSave)
+	saveAsFlag, _ := c.Doit.GetString(c.NS, flagSaveAs)
+	saveEnvFlag, _ := c.Doit.GetString(c.NS, flagSaveEnv)
+	saveEnvJSONFlag, _ := c.Doit.GetString(c.NS, flagSaveEnvJSON)
+	fetchCode := codeFlag || saveFlag || saveAsFlag != ""
+	sls := c.Serverless()
+	action, parms, err := sls.GetFunction(c.Args[0], fetchCode)
 	if err != nil {
 		return err
 	}
-	return c.PrintSandboxTextOutput(output)
+	if urlFlag {
+		host, err := sls.GetConnectedAPIHost()
+		if err != nil {
+			return err
+		}
+		_, err = fmt.Fprintln(c.Out, computeURL(action, host))
+		return err
+	} else if saveFlag || saveAsFlag != "" || saveEnvFlag != "" || saveEnvJSONFlag != "" {
+		return doSavingForFunctionGet(action, saveFlag, saveAsFlag, saveEnvFlag, saveEnvJSONFlag, parms)
+	} else if codeFlag {
+		if !*action.Exec.Binary {
+			_, err = fmt.Fprintln(c.Out, *action.Exec.Code)
+			return err
+		}
+		return errors.New("Binary code cannot be displayed on the console")
+	}
+	output := do.ServerlessOutput{Entity: action}
+	return c.PrintServerlessTextOutput(output)
 }
 
-// RunFunctionsInvoke supports the 'sandbox functions invoke' command
+// doSavingForFunctionGet performs the save operations for code and for environment variables.
+func doSavingForFunctionGet(action whisk.Action, save bool, saveAs string, saveEnv string, saveEnvJSON string,
+	parms []do.FunctionParameter) error {
+	// First process save and saveAs
+	var extension string // used only when save and !saveAs
+	var data []byte
+	if *action.Exec.Binary {
+		extension = ".zip"
+		decoded, err := base64.StdEncoding.DecodeString(*action.Exec.Code)
+		if err != nil {
+			return err
+		}
+		data = decoded
+	} else {
+		extension = fileExtensionForKind(action.Exec.Kind) // find equivalent
+		data = []byte(*action.Exec.Code)
+	}
+	if save && saveAs == "" {
+		saveAs = action.Name + extension
+	}
+	if saveAs != "" {
+		err := os.WriteFile(saveAs, data, 0666)
+		if err != nil {
+			return err
+		}
+	}
+	// Process saveEnv and saveEnvJSON.  Could do both if both are specified.
+	if saveEnv != "" || saveEnvJSON != "" {
+		keyVals := []string{}
+		envMap := map[string]string{}
+		for _, parm := range parms {
+			if parm.Init {
+				keyVal := parm.Key + "=" + parm.Value
+				keyVals = append(keyVals, keyVal)
+				envMap[parm.Key] = parm.Value
+			}
+		}
+		if saveEnv != "" {
+			data := []byte(strings.Join(keyVals, "\n"))
+			err := os.WriteFile(saveEnv, data, 0666)
+			if err != nil {
+				return err
+			}
+		}
+		if saveEnvJSON != "" {
+			data, err := json.MarshalIndent(&envMap, "", "  ")
+			if err != nil {
+				return err
+			}
+			err = os.WriteFile(saveEnvJSON, data, 0666)
+			if err != nil {
+				return err
+			}
+		}
+	}
+	return nil
+}
+
+// fileExtensionforKind finds the right file extension for a given runtime 'kind'.
+// This code will require modification when the repertoire of runtimes is extended.
+func fileExtensionForKind(kind string) string {
+	lang := strings.Split(kind, ":")[0]
+	switch strings.ToLower(lang) {
+	case "go":
+		return ".go"
+	case "nodejs":
+		return ".js"
+	case "php":
+		return ".php"
+	case "python":
+		return ".py"
+	}
+	return ""
+}
+
+// computeURL determines the URL string based on the action get output.
+// Based on code in aio-cli-plugin-runtime, src/commands/runtime/action/get.js
+func computeURL(action whisk.Action, host string) string {
+	nameParts := strings.Split(action.Namespace, "/")
+	namespace := nameParts[0]
+	var packageName string
+	if len(nameParts) > 1 {
+		packageName = nameParts[1]
+	}
+	if action.WebAction() {
+		if packageName == "" {
+			packageName = "default"
+		}
+		return fmt.Sprintf("%s/api/v1/web/%s/%s/%s", host, namespace, packageName, action.Name)
+	}
+	if packageName != "" {
+		packageName += "/"
+	}
+	return fmt.Sprintf("%s/api/v1/namespaces/%s/actions/%s%s", host, namespace, packageName, action.Name)
+}
+
+// RunFunctionsInvoke supports the 'serverless functions invoke' command
 func RunFunctionsInvoke(c *CmdConfig) error {
 	err := ensureOneArg(c)
 	if err != nil {
@@ -96,15 +221,15 @@ func RunFunctionsInvoke(c *CmdConfig) error {
 	if err != nil {
 		return err
 	}
-	output, err := SandboxExec(c, actionInvoke, args...)
+	output, err := ServerlessExec(c, actionInvoke, args...)
 	if err != nil {
 		return err
 	}
 
-	return c.PrintSandboxTextOutput(output)
+	return c.PrintServerlessTextOutput(output)
 }
 
-// RunFunctionsList supports the 'sandbox functions list' command
+// RunFunctionsList supports the 'serverless functions list' command
 func RunFunctionsList(c *CmdConfig) error {
 	argCount := len(c.Args)
 	if argCount > 1 {
@@ -120,12 +245,12 @@ func RunFunctionsList(c *CmdConfig) error {
 	if !count {
 		c.Doit.Set(c.NS, flagJSON, true)
 	}
-	output, err := RunSandboxExec(actionList, c, []string{flagCount, flagNameSort, flagNameName, flagJSON}, []string{flagLimit, flagSkip})
+	output, err := RunServerlessExec(actionList, c, []string{flagCount, flagNameSort, flagNameName, flagJSON}, []string{flagLimit, flagSkip})
 	if err != nil {
 		return err
 	}
 	if count {
-		return c.PrintSandboxTextOutput(output)
+		return c.PrintServerlessTextOutput(output)
 	}
 	// Reparse the output to use a more specific type, which can then be passed to the displayer
 	rawOutput, err := json.Marshal(output.Entity)
